﻿//******************************************************************************************************
//  DataPublisherValue.cs - Gbtc
//
//  Copyright © 2013, Grid Protection Alliance.  All Rights Reserved.
//
//  Licensed to the Grid Protection Alliance (GPA) under one or more contributor license agreements. See
//  the NOTICE file distributed with this work for additional information regarding copyright ownership.
//  The GPA licenses this file to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  04/15/2013 - Stephen C. Wills
//       Generated original version of source code.
//
//******************************************************************************************************

using System;
using System.Collections.Generic;
using SIEGate.Core;

namespace GSF.TimeSeries.Transport
{
    /// <summary>
    /// Represents a raw value supplied to the DataPublisher from its output adapters.
    /// </summary>
    public class DataPublisherValue : IMeasurementValue
    {
        #region [ Members ]

        // Fields
        private ICollection<CompactMeasurement> m_compactMeasurements;
        private ICollection<BufferBlockMeasurement> m_bufferBlockMeasurements;

        #endregion

        #region [ Constructors ]

        /// <summary>
        /// Creates a new instance of the <see cref="DataPublisherValue"/> class.
        /// </summary>
        /// <param name="compactMeasurements">Collection of compact measurements.</param>
        /// <param name="bufferBlockMeasurements">Collection of buffer block measurements.</param>
        public DataPublisherValue(ICollection<CompactMeasurement> compactMeasurements, ICollection<BufferBlockMeasurement> bufferBlockMeasurements)
        {
            m_compactMeasurements = compactMeasurements;
            m_bufferBlockMeasurements = bufferBlockMeasurements;
        }

        #endregion

        #region [ Methods ]

        /// <summary>
        /// Returns the raw value. The value is an array of <see cref="CompactMeasurement"/>s.
        /// </summary>
        /// <returns>The raw value.</returns>
        public dynamic GetRaw()
        {
            return Tuple.Create(m_compactMeasurements, m_bufferBlockMeasurements);
        }

        #endregion
    }
}
