﻿/*****************************************************************************\
 *             University of Illinois/NCSA Open Source License
 *
 * 
 *      Copyright © 2012,2013, The Board of Trustees of the University of Illinois. All rights reserved.
 *
 *      Developed by: Rakesh Bobba, Derek Dagit, Erich Heine, Denis Kholine,
 *                    Klara Nahrstedt, David Rodgers,   William H. Sanders, Tim Yardley
 *
 *      Information Trust Institute/Coordinated Science Laboratory
 * 
 *      University of Illinois
 *      
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      http://www.iti.illinois.edu / http://csl.illinois.edu
 * 
 *      Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
 *      associated documentation files (the “Software”), to deal with the Software without restriction, 
 *      including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *      and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
 *      subject to the following conditions:
 *
 *      Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *      following disclaimers.
 *      
 *      Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *      the following disclaimers in the documentation and/or other materials provided with the distribution.
 *
 *      Neither the names of Information Trust institute, Coordinated Science Laboratory, University of Illinois,
 *      The Board of Trustees of the University of Illinois nor the names of its contributors may be used to 
 *      endorse or promote products derived from this Software without specific prior written permission.
 *      
 *      THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 *      LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN
 *      NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *      WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 *      SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *      
 \*****************************************************************************/

 using System;
using System.Collections.Generic;

namespace SIEGate.Core
{
    public delegate void MeasurementPasser(params IMeasurement[] m);

    public interface IAdapter : INotificationTarget
    {
        MeasurementPasser Emit { get; set; }

        ISet<Guid> IncomingStreamIDs
        {
            get;
        }

        ISet<Guid> OutgoingStreamIDs
        {
            get;
        }


        IAdapterInfo Info
        {
            get;
            set;
        }

        // Random thought.
        // Depending on what the 3rd party devs want to do as far as transforming the data, we may or may not want the operation to spawn a thread.
        // At the same time, we would like the 3rd party devs not to have to know or care about any thread-safe, QoS-amenable class heirarchies, APIs, etc.
        // What if we defined our own custom attribute classes that let 3rd party devs tell us, in a programmatic way, whether their collect() deserves to launch its own threads or if it is so short and sweet that it can do without all the overhead?
        // Devs are likely already familiar with 'inline' and other such things, so one vision of usage could be:
        // [SimpleStuff]
        // public override void collect(IMeasurement input) ...
        // This would give us a way of choosing at runtime what to do re: threading.
        // 2012.08.01 - dagit
        void Collect(params IMeasurement[] input);

        void link(IAdapter next);
    }

    public interface ISmartAdapter : IAdapter
    {
        ISet<Guid> GetInputsFor(ISet<Guid> outputs);

        event EventHandler ChangeInputs;
    }
}
