//******************************************************************************************************
//  ServiceHelperLogger.cs - Gbtc
//
//  Copyright  2013, Grid Protection Alliance.  All Rights Reserved.
//
//  Licensed to the Grid Protection Alliance (GPA) under one or more contributor license agreements. See
//  the NOTICE file distributed with this work for additional information regarding copyright ownership.
//  The GPA licenses this file to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  05/13/2013 - Erich A. Heine & Stephen C. Wills
//       Generated original version of source code.
//
//******************************************************************************************************

/*****************************************************************************\
 *             University of Illinois/NCSA Open Source License
 *
 *
 *      Copyright  2012,2013, The Board of Trustees of the University of Illinois. All rights reserved.
 *
 *      Developed by: Rakesh Bobba, Derek Dagit, Erich Heine, Denis Kholine,
 *                    Klara Nahrstedt, David Rodgers,   William H. Sanders, Tim Yardley
 *
 *      Information Trust Institute/Coordinated Science Laboratory
 * 
 *      University of Illinois
 *      
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: ServiceHelperLogger.cs
 *
 *      Authors: Erich Heine, David Rodgers, Denis Kholine, Derek Dagit,
 *      William H. Sanders, Klara Nahrstedt, Rakesh Bobba, Tim Yardley
 *      Creation Date: 2012
 *
 *      Copyright (c) 2012 William H. Sanders, Klara Nahrstedt, Erich Heine,
 *      Rakesh Bobba, Tim Yardley,
 *      and The Board of Trustees of the University of Illinois
 *      All rights reserved.
 * 
 *      THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 *      LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN
 *      NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *      WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 *      SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *
 \*****************************************************************************/

using System;
using GSF;
using GSF.ServiceProcess;

namespace SIEGate.Core.Logging
{
    public class ServiceHelperLogger : Logger
    {
        private ServiceHelper m_serviceHelper;

        public ServiceHelperLogger(ServiceHelper serviceHelper)
        {
            m_serviceHelper = serviceHelper;
        }

        public override void Log(LogLevel level, DateTime time, string source, string tag, string message)
        {
            UpdateType updateType = UpdateType.Alarm;

            switch (level)
            {
                case LogLevel.Debug:
                case LogLevel.Info:
                case LogLevel.Verbose:
                    updateType = UpdateType.Information;
                    break;

                case LogLevel.Warn:
                    updateType = UpdateType.Warning;
                    break;

                case LogLevel.Error:
                case LogLevel.Fatal:
                case LogLevel.Unexpected:
                    updateType = UpdateType.Alarm;
                    break;

                default:
                    message = string.Format("We got some sort of log level that doesn't exist! :: {0}", message);
                    break;
            }

            m_serviceHelper.UpdateStatus(updateType, string.Format("{0:G} {1:u}: {2}: {3}: {4}{5}{5}", level, time, source, tag, message, Environment.NewLine));
        }

        public override void LogException(LogLevel level, DateTime time, string source, string tag, Exception ex)
        {
            base.LogException(level, time, source, tag, ex);

            if ((object)m_serviceHelper.ErrorLogger != null)
                m_serviceHelper.ErrorLogger.Log(ex);
        }
    }
}
