﻿/*****************************************************************************\
 *             University of Illinois/NCSA Open Source License
 *
 * 
 *      Copyright © 2012,2013, The Board of Trustees of the University of Illinois. All rights reserved.
 *
 *      Developed by: Rakesh Bobba, Derek Dagit, Erich Heine, Denis Kholine,
 *                    Klara Nahrstedt, David Rodgers,   William H. Sanders, Tim Yardley
 *
 *      Information Trust Institute/Coordinated Science Laboratory
 * 
 *      University of Illinois
 *      
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      http://www.iti.illinois.edu / http://csl.illinois.edu
 * 
 *      Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
 *      associated documentation files (the “Software”), to deal with the Software without restriction, 
 *      including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *      and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
 *      subject to the following conditions:
 *
 *      Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *      following disclaimers.
 *      
 *      Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *      the following disclaimers in the documentation and/or other materials provided with the distribution.
 *
 *      Neither the names of Information Trust institute, Coordinated Science Laboratory, University of Illinois,
 *      The Board of Trustees of the University of Illinois nor the names of its contributors may be used to 
 *      endorse or promote products derived from this Software without specific prior written permission.
 *      
 *      THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 *      LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN
 *      NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *      WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 *      SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *      
 \*****************************************************************************/

using GSF;
using System;
using SIEGate.Core.Legacy;

namespace SIEGate.Core.Measurements
{
    [Serializable]
    public class RawData : IMeasurement
    {
        public Guid StreamID
        {
            get;
            private set;
        }

        public Ticks Timestamp
        {
            get;
            private set;
        }

        public MeasurementStateFlags StateFlags
        {
            get;
            set;
        }

        public IMeasurementValue Value
        {
            get;
            private set;
        }

        public IMeasurement Copy()
        {
            return new RawData(this);
        }

        public MeasurementKey Key
        {
            get;
            set;
        }

        public string TagName
        {
            get;
            set;
        }

        public double AdjustedValue
        {
            get
            {
                // BUG: Only works for numerical values.  Value can store strings.
                return Value.GetRaw() * Multiplier + Adder;
            }
        }

        public double Adder
        {
            get;
            set;
        }

        public double Multiplier
        {
            get;
            set;
        }

        public Ticks ReceivedTimestamp
        {
            get;
            set;
        }

        public Ticks PublishedTimestamp
        {
            get;
            set;
        }

        public MeasurementValueFilterFunction MeasurementValueFilter
        {
            get;
            set;
        }

        RawData(RawData other)
        {
            StreamID = other.StreamID;
            Timestamp = other.Timestamp;
            StateFlags = other.StateFlags;
            Value = other.Value;
        }

        public RawData(Guid streamID, IMeasurementValue value)
        {
            StreamID = streamID;
            Timestamp = 0L;
            StateFlags = MeasurementStateFlags.Normal;
            Value = value;
        }

        public RawData(Guid streamID, IMeasurementValue value, Ticks nTicks) : this(streamID, value)
        {
            Timestamp = nTicks;
        }
    }
}
