﻿//******************************************************************************************************
//  BufferBlockValue.cs - Gbtc
//
//  Copyright © 2013, Grid Protection Alliance.  All Rights Reserved.
//
//  Licensed to the Grid Protection Alliance (GPA) under one or more contributor license agreements. See
//  the NOTICE file distributed with this work for additional information regarding copyright ownership.
//  The GPA licenses this file to you under the Eclipse Public License -v 1.0 (the "License"); you may
//  not use this file except in compliance with the License. You may obtain a copy of the License at:
//
//      http://www.opensource.org/licenses/eclipse-1.0.php
//
//  Unless agreed to in writing, the subject software distributed under the License is distributed on an
//  "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the
//  License for the specific language governing permissions and limitations.
//
//  Code Modification History:
//  ----------------------------------------------------------------------------------------------------
//  05/07/2013 - Stephen C. Wills
//       Generated original version of source code.
//
//******************************************************************************************************

using System;
using GSF;

namespace SIEGate.Core.Measurements.Values
{
    public class BufferBlockValue : IMeasurementValue
    {
        #region [ Members ]

        // Fields
        private Tuple<byte[], int> m_bufferBlock;

        #endregion

        #region [ Constructors ]

        public BufferBlockValue(byte[] buffer, int offset, int length)
        {
            byte[] poolBuffer = BufferPool.TakeBuffer(length);
            Buffer.BlockCopy(buffer, offset, poolBuffer, 0, length);
            m_bufferBlock = Tuple.Create(buffer, length);
        }

        ~BufferBlockValue()
        {
            if ((object)m_bufferBlock != null)
            {
                BufferPool.ReturnBuffer(m_bufferBlock.Item1);
                m_bufferBlock = null;
            }
        }

        #endregion

        #region [ Methods ]

        public dynamic GetRaw()
        {
            return m_bufferBlock;
        }

        #endregion
    }
}
