/*****************************************************************************\
 *             University of Illinois/NCSA Open Source License
 *
 * 
 *      Copyright © 2012,2013, The Board of Trustees of the University of Illinois. All rights reserved.
 *
 *      Developed by: Rakesh Bobba, Derek Dagit, Erich Heine, Denis Kholine,
 *                    Klara Nahrstedt, David Rodgers,   William H. Sanders, Tim Yardley
 *
 *      Information Trust Institute/Coordinated Science Laboratory
 * 
 *      University of Illinois
 *      
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      http://www.iti.illinois.edu / http://csl.illinois.edu
 * 
 *      Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
 *      associated documentation files (the “Software”), to deal with the Software without restriction, 
 *      including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *      and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
 *      subject to the following conditions:
 *
 *      Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *      following disclaimers.
 *      
 *      Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *      the following disclaimers in the documentation and/or other materials provided with the distribution.
 *
 *      Neither the names of Information Trust institute, Coordinated Science Laboratory, University of Illinois,
 *      The Board of Trustees of the University of Illinois nor the names of its contributors may be used to 
 *      endorse or promote products derived from this Software without specific prior written permission.
 *      
 *      THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 *      LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN
 *      NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *      WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 *      SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *      
 \*****************************************************************************/

﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace SIEGate.Core.MessageBusses
{
    class RingBufferMessageBusProxy : IMessageBusProxy
    {
        private RingBufferMessageBus mb;
        private ISet<Guid> subscribed = new HashSet<Guid>();
        private ISet<Guid> published = new HashSet<Guid>();
        internal RingBufferMessageBusConsumer Consumer;
        internal RingBufferMessageBusProducer Producer;

        public void Subscribe(ISet<Guid> neededMessages)
        {
            subscribed.UnionWith(neededMessages);
            Consumer.Enable();
        }

        public void UnSubscribe(ISet<Guid> toRemove)
        {
            foreach (Guid r in subscribed.Intersect(toRemove))
            {
                subscribed.Remove(r);
            }
            if (!subscribed.Any())
            {
                Consumer.Disable();
            }
        }

        public void Publish(ISet<Guid> providedMessages)
        {
            published.UnionWith(providedMessages);
        }

        public void UnPublish(ISet<Guid> toRemove)
        {
            foreach (Guid r in published.Intersect(toRemove))
            {
                published.Remove(r);
            }
        }


        public void Send(IMeasurement m)
        {
            Producer.send(m);
        }
        public void Send(IMeasurement[] m)
        {
            Producer.send(m);
        }

        public IMeasurement _Recv()
        {
            return Consumer.Recv();
        }
        public IMeasurement[] Recv(int max)
        {
            return Consumer.RecvMany(max);
        }

        internal RingBufferMessageBusProxy(RingBufferMessageBus mb)
        {
            this.mb = mb;
            Consumer = this.mb.GetConsumer(subscribed);
            Producer = this.mb.GetProducer(published, new QOSParams());
        }

        public event EventHandler<MessageBusSubscriptionEventArgs> EndSubscription;
        public event EventHandler<MessageBusSubscriptionEventArgs> StartSubscription;

        public ISet<Guid> Published
        {
            get
            {
                return published;
            }
        }

        public ISet<Guid> Subscribed
        {
            get
            {
                return subscribed;
            }
        }

        public ISet<Guid> ActivePubs
        {
            get
            {
                // TODO: this is broken, when we revisit this thing, fix it
                return published;
            }
        }
    }
}
