/*****************************************************************************\
 *             University of Illinois/NCSA Open Source License
 *
 * 
 *      Copyright © 2012,2013, The Board of Trustees of the University of Illinois. All rights reserved.
 *
 *      Developed by: Rakesh Bobba, Derek Dagit, Erich Heine, Denis Kholine,
 *                    Klara Nahrstedt, David Rodgers,   William H. Sanders, Tim Yardley
 *
 *      Information Trust Institute/Coordinated Science Laboratory
 * 
 *      University of Illinois
 *      
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      http://www.iti.illinois.edu / http://csl.illinois.edu
 * 
 *      Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
 *      associated documentation files (the “Software”), to deal with the Software without restriction, 
 *      including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *      and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
 *      subject to the following conditions:
 *
 *      Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *      following disclaimers.
 *      
 *      Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *      the following disclaimers in the documentation and/or other materials provided with the distribution.
 *
 *      Neither the names of Information Trust institute, Coordinated Science Laboratory, University of Illinois,
 *      The Board of Trustees of the University of Illinois nor the names of its contributors may be used to 
 *      endorse or promote products derived from this Software without specific prior written permission.
 *      
 *      THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 *      LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN
 *      NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *      WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 *      SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *      
 \*****************************************************************************/

﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace SIEGate.Core.MessageBusses
{
    class SimpleMessageBusProxy : IMessageBusProxy
    {
        private SimpleMessageBus mb;
        private QOSParams qosparams;
        private ISet<Guid> subscribed = new HashSet<Guid>();
        private ISet<Guid> published = new HashSet<Guid>();
        public event EventHandler<MessageBusSubscriptionEventArgs> StartSubscription;
        public event EventHandler<MessageBusSubscriptionEventArgs> EndSubscription;

        public void Subscribe(ISet<Guid> neededMessages)
        {
            foreach (Guid a in neededMessages)
            {
                subscribed.Add(a);
            }
        }

        public ISet<Guid> Published
        {
            get
            {
                return published;
            }
        }
        public ISet<Guid> Subscribed
        {
            get
            {
                return subscribed;
            }
        }
        public ISet<Guid> ActivePubs
        {
            get
            {
                return subscribed;
            }
        }

        public void UnSubscribe(ISet<Guid> toRemove)
        {
            foreach (Guid r in subscribed.Intersect(toRemove))
            {
                subscribed.Remove(r);
            }
        }

        public void Publish(ISet<Guid> providedMessages)
        {
            foreach (Guid a in providedMessages)
            {
                published.Add(a);
            }
        }

        public void UnPublish(ISet<Guid> toRemove)
        {
            foreach (Guid r in published.Intersect(toRemove))
            {
                published.Remove(r);
            }
        }

        public void Send(IMeasurement m)
        {
            Routable r = new Routable(m, this.qosparams);
            mb.Put(r);
        }

        public void Send(IMeasurement[] m)
        {
            foreach (IMeasurement i in m)
            {
                Send(i);
            }
        }

        private IMeasurement _Recv()
        {
            IRoutable res = this.mb.Get(this.subscribed);
            return res == null ? null : res.data;
        }

        public IMeasurement[] Recv(int max)
        {
            IMeasurement m = _Recv();
            return m != null ? new IMeasurement[1] { m } : new IMeasurement[0];
        }

        internal SimpleMessageBusProxy(SimpleMessageBus messageBus)
        {
            mb = messageBus;
            // TODO: HACK: Fix qos params etc
            qosparams = new QOSParams();
        }
    }
}
