﻿#region [ Modification History ]
/*
 * 07/11/2013 Denis Kholine
 *  Add license and description.
 */
#endregion

#region  [ UIUC NCSA Open Source License ]
/*****************************************************************************\
 *             University of Illinois/NCSA Open Source License
 *
 * 
 *      Copyright © 2012,2013, The Board of Trustees of the University of Illinois. All rights reserved.
 *
 *      Developed by: Rakesh Bobba, Derek Dagit, Erich Heine, Denis Kholine,
 *                    Klara Nahrstedt, David Rodgers,   William H. Sanders, Tim Yardley
 *
 *      Information Trust Institute/Coordinated Science Laboratory
 * 
 *      University of Illinois
 *      
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      http://www.iti.illinois.edu / http://csl.illinois.edu
 * 
 *      Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
 *      associated documentation files (the “Software”), to deal with the Software without restriction, 
 *      including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *      and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
 *      subject to the following conditions:
 *
 *      Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *      following disclaimers.
 *      
 *      Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *      the following disclaimers in the documentation and/or other materials provided with the distribution.
 *
 *      Neither the names of Information Trust institute, Coordinated Science Laboratory, University of Illinois,
 *      The Board of Trustees of the University of Illinois nor the names of its contributors may be used to 
 *      endorse or promote products derived from this Software without specific prior written permission.
 *      
 *      THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 *      LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN
 *      NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *      WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 *      SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *      
 \*****************************************************************************/
#endregion

#region [ Using ]
using System;
using System.Data;
using System.Linq;
using System.Diagnostics;
using SIEGate.Core;
using SIEGate.Core.Logging;
using SIEGate.Core.Fakes;
using SIEGate.Core.Notifications;
using SIEGate.Core.MessageBusses;
using SIEGate.Core.Configurator;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using GSF;
using GSF.TimeSeries.Transport;
#endregion

namespace SIEGateCoreTest.Logging
{
    /// <summary>
    /// A test class for SIEGate.Core.Logging.ConsoleLogger
    /// </summary>
    [TestClass]
    public class ConsoleLoggerTests
    {
        #region [ Members ]
        private ConsoleLogger target;
        #endregion

        #region [ Methods ]
        /// <summary>
        /// A test for MinLogLevel
        /// </summary>
        [TestMethod]
        [TestCategory("ConsoleLogger")]
        public void MinLogLevelTest()
        {
            bool expected = true;
            bool actual = false;
            target = new ConsoleLogger();

            LogLevel level = LogLevel.Verbose;
            target.MinLogLevel = level;

            actual = (target.MinLogLevel.CompareTo(level) == 0);
            Assert.AreEqual(expected, actual);
        }


        /// <summary>
        /// A test for Log
        /// </summary>
        [TestMethod]
        [TestCategory("ConsoleLogger")]
        public void LogTest()
        {
            bool expected = true;
            bool actual = false;
            target = new ConsoleLogger();

            LogLevel level = LogLevel.Verbose;
            DateTime time = new DateTime(1000);
            string source = "Source";
            string tag = "Tag";
            string message = "Message";
            target.Log(level, time, source, tag, message);

            actual = (target.MinLogLevel.CompareTo(level) <=0);

            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// A test for LogException
        /// </summary>
        [TestMethod]
        [TestCategory("ConsoleLogger")]
        public void LogExceptionTest()
        {
            bool expected = true;
            bool actual = false;
            target = new ConsoleLogger();

            LogLevel level = LogLevel.Verbose;
            DateTime time = new DateTime(1000);
            string source = "Source";
            string tag = "Tag";
            Exception exception = new Exception();
            
            target.LogException(level, time, source, tag,exception);

            actual = (target.MinLogLevel.CompareTo(level) <= 0);

            Assert.AreEqual(expected, actual);
        }


        /// <summary>
        /// A test for Console Logger
        /// </summary>
        [TestMethod]
        [TestCategory("ConsoleLogger")]
        public void ConsoleLoggerTest()
        {
            bool expected = true;
            bool actual = false;

            try
            {
                target = new ConsoleLogger();
                actual = true;
            }
            catch
            {
                actual = false;
            } 
            Assert.AreEqual(expected, actual);
        }
        #endregion
    }
}
