﻿#region [ Modification History ]
/*
 * 08/19/2013 Denis Kholine
 *  Add comments and license
 */
#endregion

#region  [ UIUC NCSA Open Source License ]
/*****************************************************************************\
 *             University of Illinois/NCSA Open Source License
 *
 * 
 *      Copyright © 2012,2013, The Board of Trustees of the University of Illinois. All rights reserved.
 *
 *      Developed by: Rakesh Bobba, Derek Dagit, Erich Heine, Denis Kholine,
 *                    Klara Nahrstedt, David Rodgers,   William H. Sanders, Tim Yardley
 *
 *      Information Trust Institute/Coordinated Science Laboratory
 * 
 *      University of Illinois
 *      
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      http://www.iti.illinois.edu / http://csl.illinois.edu
 * 
 *      Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
 *      associated documentation files (the “Software”), to deal with the Software without restriction, 
 *      including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *      and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
 *      subject to the following conditions:
 *
 *      Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *      following disclaimers.
 *      
 *      Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *      the following disclaimers in the documentation and/or other materials provided with the distribution.
 *
 *      Neither the names of Information Trust institute, Coordinated Science Laboratory, University of Illinois,
 *      The Board of Trustees of the University of Illinois nor the names of its contributors may be used to 
 *      endorse or promote products derived from this Software without specific prior written permission.
 *      
 *      THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 *      LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN
 *      NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *      WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 *      SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *      
 \*****************************************************************************/
#endregion

#region [ Using ]
using GSF;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Data;
using System.Text;
using System.Threading.Tasks;
using SIEGate.Core;
using SIEGate.Core.Adapters;
using SIEGate.Core.Stats;
using SIEGate.Core.Fakes;
using SIEGate.Core.Notifications;
using SIEGate.Core.Measurements;
using SIEGate.Core.Measurements.Values;
using SIEGate.Core.Logging;
using SIEGate.Core.Legacy;
using SIEGate.Core.MessageBusses;
using GSF.TimeSeries.Transport;
using Disruptor;
using Microsoft.VisualStudio.TestTools.UnitTesting;
#endregion


namespace SIEGateCoreTest.MessageBusTests
{
    /// <summary>
    /// A test class for RingBufferMessageBusConsumer
    /// </summary>
    [TestClass]
    public class RingBufferMessageBusConsumerTests
    {
        #region [ Members ]
        private MockMeasurement measurement;
        private RingBufferMessageBusProducer producer;
        private RingBufferMessageBusConsumer consumer;
        private ISet<Guid> c_streamIDs;
        private ISet<Guid> p_streamIDs;
        private RingBufferMessageBus mb;
        private ISequenceBarrier front;
        private long startseq;
        private RingBuffer<Routable> rb;
        private QOSParams qos;
        private int bufsize;
        private Func<Routable> eventFactory;
        private Sequence[] sequence; 
        #endregion


        #region [ Methods ]
        /// <summary>
        /// A test for RingBufferMessageBusConsumer constructor
        /// </summary>
        [TestMethod]
        [TestCategory("RingBufferMessageBusConsumer")]
        public void RingBufferMessageBusConsumerTest()
        {
            // Initialize ring buffer and message bus
            c_streamIDs = new HashSet<Guid>();
            p_streamIDs = new HashSet<Guid>();

            mb = new RingBufferMessageBus(4);

            p_streamIDs.Add(Guid.NewGuid());
            c_streamIDs.Add(Guid.NewGuid());

            qos = new QOSParams();

            sequence = new Sequence[2] { new Sequence(10), new Sequence(20) };

            measurement = new MockMeasurement(Guid.NewGuid());
            rb = new RingBuffer<Routable>(() => new Routable(), 4);

            rb.NewBarrier(sequence);
            rb.NewBarrier(sequence);

            rb.SetGatingSequences(sequence);
            // Generate consumer
            startseq = 2;

            mb.RingBuffer.NewBarrier(sequence);

            producer = new RingBufferMessageBusProducer(p_streamIDs, qos, rb);
            consumer = new RingBufferMessageBusConsumer(c_streamIDs, mb, front, startseq);

            mb.StartConsumer(consumer);

            Assert.IsNotNull(consumer);
            Assert.IsInstanceOfType(consumer, typeof(RingBufferMessageBusConsumer));
        }

 
        /// <summary>
        /// A test for RecvMany field
        /// </summary>
        [TestMethod]
        [TestCategory("RingBufferMessageBusConsumer")]
        public void RingBufferMessageBusConsumerRecvManyTest()
        {
            // Initialize ring buffer and message bus
            c_streamIDs = new HashSet<Guid>();
            p_streamIDs = new HashSet<Guid>();
            
            mb = new RingBufferMessageBus(4);

            p_streamIDs.Add(Guid.NewGuid());
            c_streamIDs.Add(Guid.NewGuid());

            qos = new QOSParams();
            
            sequence = new Sequence[2] { new Sequence(10), new Sequence(20) };

            measurement = new MockMeasurement(Guid.NewGuid());
            rb = new RingBuffer<Routable>(() => new Routable(), 4);
            
            rb.NewBarrier(sequence);
            rb.NewBarrier(sequence);
            
            rb.SetGatingSequences(sequence);
            // Generate consumer
            startseq = 2;

            

            mb.RingBuffer.NewBarrier(sequence);

            producer = new RingBufferMessageBusProducer(p_streamIDs, qos, rb);
            consumer = new RingBufferMessageBusConsumer(c_streamIDs, mb, front, startseq);

            mb.StartConsumer(consumer);
 
            consumer.RecvMany(3);
        }

        /// <summary>
        /// A test for Recv property
        /// </summary>
        [TestMethod]
        [TestCategory("RingBufferMessageBusConsumer")]
        public void RingBufferMessageBusConsumerRecvTest()
        {
        }

        /// <summary>
        /// A test for Sequence property
        /// </summary>
        [TestMethod]
        [TestCategory("RingBufferMessageBusConsumer")]
        public void RingBufferMessageBusConsumerSequenceTest()
        {
        }

        /// <summary>
        /// A test for Disable method
        /// </summary>
        [TestMethod]
        [TestCategory("RingBufferMessageBusConsumer")]
        public void RingBufferMessageBusConsumerDisableTest()
        {
        }
        #endregion
    }
}
